# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of Hooke.
#
# Hooke is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Hooke is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
# Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Hooke.  If not, see
# <http://www.gnu.org/licenses/>.

"""The ``command_stack`` module provides tools for managing and
executing stacks of :class:`~hooke.engine.CommandMessage`\s.
"""


class CommandStack (list):
    """Store a stack of commands.

    Examples
    --------
    >>> from .engine import CommandMessage

    Define two dummy commands for testing.

    >>> class CommandA (object):
    ...     name = 'CommandA'
    >>> ca = CommandA()
    >>> class CommandB (CommandA):
    ...     name = 'CommandB'
    >>> cb = CommandB()

    Show off `CommandStack`\s functionality.

    >>> c = CommandStack([CommandMessage(ca, {'param':'A'})])
    >>> c.append(CommandMessage(cb, {'param':'B'}))
    >>> c.append(CommandMessage(ca, {'param':'C'}))
    >>> c.append(CommandMessage(cb, {'param':'D'}))

    Implement a dummy :meth:`_execute` for testing.
    
    >>> def execute(hooke, command_message):
    ...     cm = command_message
    ...     print 'EXECUTE', cm.command.name, cm.arguments
    >>> c._execute = execute

    >>> c.execute()  # doctest: +ELLIPSIS
    EXECUTE CommandA {'param': 'A'}
    EXECUTE CommandB {'param': 'B'}
    EXECUTE CommandA {'param': 'C'}
    EXECUTE CommandB {'param': 'D'}

    :meth:`filter` allows you to select which commands get executed.
    If, for example, you are applying a set of commands to the current
    :class:`~hooke.curve.Curve`, you may only want to execute
    instances of :class:`~hooke.plugin.curve.CurveCommand`.  Here we
    only execute instances of `CommandB`.
    
    >>> def filter(command_message):
    ...     return isinstance(command_message.command, CommandB)
    >>> c.filter = filter

    Apply the stack to the current curve
    >>> c.execute()  # doctest: +ELLIPSIS
    EXECUTE CommandB {'param': 'B'}
    EXECUTE CommandB {'param': 'D'}
    """
    def execute(self, hooke):
        """Execute a stack of commands.

        See Also
        --------
        _execute, filter
        """
        for command_message in self:
            if self.filter(command_message) == True:
                self._execute(hooke, command_message)

    def filter(self, command_message):
        """Any commands in the stack that are not subclasses of
        :class:`~hooke.plugin.curve.CurveCommand` are ignored.
        """
        return True

    def _execute(self, hooke, command_message):
        hooke.run_command(command_message.command, command_message.arguments)
