# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of Hooke.
#
# Hooke is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Hooke is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
# Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Hooke.  If not, see
# <http://www.gnu.org/licenses/>.

"""The `engine` module provides :class:`CommandEngine` for executing
:class:`hooke.command.Command`\s.
"""

import logging

from .command import NullQueue


class QueueMessage (object):
    def __str__(self):
        return self.__class__.__name__


class CloseEngine (QueueMessage):
    pass


class CommandMessage (QueueMessage):
    """A message storing a command to run, `command` should be a
    :class:`hooke.command.Command` instance, and `arguments` should be
    a :class:`dict` with `argname` keys and `value` values to be
    passed to the command.
    """
    def __init__(self, command, arguments=None):
        self.command = command
        if arguments == None:
            arguments = {}
        self.arguments = arguments


class CommandEngine (object):
    def run(self, hooke, ui_to_command_queue, command_to_ui_queue):
        """Get a :class:`QueueMessage` from the incoming
        `ui_to_command_queue` and act accordingly.

        If the message is a :class:`CommandMessage` instance, the
        command run may read subsequent input from
        `ui_to_command_queue` (if it is an interactive command).  The
        command may also put assorted data into `command_to_ui_queue`.

        When the command completes, it will put a
        :class:`hooke.command.CommandExit` instance into
        `command_to_ui_queue`, at which point the `CommandEngine` will
        be ready to receive the next :class:`QueueMessage`.
        """
        log = logging.getLogger('hooke')
        while True:
            log.debug('engine waiting for command')
            msg = ui_to_command_queue.get()
            if isinstance(msg, CloseEngine):
                command_to_ui_queue.put(hooke)
                log.debug(
                    'engine closing, placed hooke instance in return queue')
                break
            assert isinstance(msg, CommandMessage), type(msg)
            log.debug('engine running %s' % msg.command.name)
            msg.command.run(hooke, ui_to_command_queue, command_to_ui_queue,
                            **msg.arguments)

    def run_command(self, hooke, command, arguments):
        """Internal command execution.

        This allows commands to execute sub commands and enables
        :class:`~hooke.command_stack.CommandStack` execution.

        Note that these commands *do not* have access to the usual UI
        communication queues, so make sure they will not need user
        interaction.
        """
        command.run(hooke, NullQueue(), NullQueue(), arguments)
